@@ -126,7 +126,7 @@ static int do_uninstall(Options *op);
 
 static int sanity_check_backup_log_entries(Options *op, BackupInfo *b);
 
-
+static char *create_backwards_compatible_version_string(const char *str);
 
 
 
@@ -141,6 +141,8 @@ static int sanity_check_backup_log_entries(Options *op, BackupInfo *b);
 
 int init_backup(Options *op, Package *p)
 {
+    mode_t orig_mode;
+    char *version;
     FILE *log;
     
     /* remove the directory, if it already exists */
@@ -157,9 +159,22 @@ int init_backup(Options *op, Package *p)
         return FALSE;
     }
 
+    /*
+     * fopen below creates the file with mode 0666 & ~umask.
+     * In order to ensure the result of that calculation is BACKUP_LOG_PERMS,
+     * we temporarily set umask to ~BACKUP_LOG_PERMS to leave just the bits
+     * we want.
+     *
+     * This assumes that BACKUP_LOG does not already exist (if it does, the
+     * file permissions will not be modified.) This is assured by the
+     * directory removal and re-creation code above.
+     */
+    orig_mode = umask(~BACKUP_LOG_PERMS);
+ 
     /* create the log file */
     
     log = fopen(BACKUP_LOG, "a");
+    umask(orig_mode);
     if (!log) {
         ui_error(op, "Unable to create backup log file '%s' (%s).",
                  BACKUP_LOG, strerror(errno));
@@ -168,9 +183,13 @@ int init_backup(Options *op, Package *p)
 
     /* write the version and description */
 
-    fprintf(log, "%s\n", p->version_string);
-    fprintf(log, "%s\n", p->description);
+    version = create_backwards_compatible_version_string(p->version);
     
+    fprintf(log, "%s\n", version);
+    fprintf(log, "%s\n", p->description);
+
+    nvfree(version);
+        
     /* close the log file */
 
     if (fclose(log) != 0) {
@@ -179,14 +198,6 @@ int init_backup(Options *op, Package *p)
         return FALSE;
     }
 
-    /* set the log file permissions */
-    
-    if (chmod(BACKUP_LOG, BACKUP_LOG_PERMS) == -1) {
-        ui_error(op, "Unable to set file permissions %04o for %s (%s).",
-                 BACKUP_LOG_PERMS, BACKUP_LOG, strerror(errno));
-        return FALSE;
-    }
-
     return TRUE;
     
 } /* init_backup() */
@@ -494,9 +505,9 @@ static int do_uninstall(Options *op)
         "Your driver installation has been "
         "altered since it was initially installed; this may happen, "
         "for example, if you have since installed the NVIDIA driver through "
-        "a mechanism other than the nvidia-installer (such as rpm or "
-        "with the NVIDIA tarballs).  The nvidia-installer will "
-        "attempt to uninstall as best it can.";
+        "a mechanism other than nvidia-installer (such as your "
+        "distribution's native package management system).  "
+        "nvidia-installer will attempt to uninstall as best it can.";
     
     /* do we even have a backup directory? */
 
@@ -515,13 +526,15 @@ static int do_uninstall(Options *op)
                     existing_installation_is_borked, op->log_file_name);
 
         } else {
-            ui_warn(op, existing_installation_is_borked);
+            ui_warn(op, "%s", existing_installation_is_borked);
         }
     }
     
     tmpstr = nvstrcat("Uninstalling ", b->description, " (",
                       b->version, "):", NULL);
-   
+
+    run_distro_hook(op, "pre-uninstall");
+
     ui_status_begin(op, tmpstr, "Uninstalling");
 
     free(tmpstr);
@@ -618,14 +631,14 @@ static int do_uninstall(Options *op)
             if (!nvrename(op, tmpstr, e->filename)) {
                 ui_warn(op, "Unable to restore file '%s'.", e->filename);
             } else {
-                if (chmod(e->filename, e->mode) == -1) {
-                    ui_warn(op, "Unable to restore permissions %04o for "
-                            "file '%s'.", e->mode, e->filename);
+                if (chown(e->filename, e->uid, e->gid)) {
+                    ui_warn(op, "Unable to restore owner (%d) and group "
+                            "(%d) for file '%s' (%s).",
+                            e->uid, e->gid, e->filename, strerror(errno));
                 } else {
-                    if (chown(e->filename, e->uid, e->gid)) {
-                        ui_warn(op, "Unable to restore owner (%d) and group "
-                                "(%d) for file '%s' (%s).",
-                                e->uid, e->gid, e->filename, strerror(errno));
+                    if (chmod(e->filename, e->mode) == -1) {
+                        ui_warn(op, "Unable to restore permissions %04o for "
+                                "file '%s'.", e->mode, e->filename);
                     }
                 }
             }
@@ -643,6 +656,8 @@ static int do_uninstall(Options *op)
         /* XXX what to do if this fails?... nothing */
     }
 
+    run_distro_hook(op, "post-uninstall");
+
     return TRUE;
     
 } /* do_uninstall() */
@@ -659,7 +674,7 @@ static BackupInfo *read_backup_log_file(Options *op)
 {
     struct stat stat_buf;
     char *buf, *c, *line, *filename;
-    int fd, num, line_num = 0;
+    int fd, num, length, line_num = 0;
     float percent;
     
     BackupLogEntry *e;
@@ -698,7 +713,9 @@ static BackupInfo *read_backup_log_file(Options *op)
 
     /* map the file */
 
-    buf = mmap(0, stat_buf.st_size, PROT_READ, MAP_FILE | MAP_SHARED, fd, 0);
+    length = stat_buf.st_size;
+
+    buf = mmap(0, length, PROT_READ, MAP_FILE | MAP_SHARED, fd, 0);
     if (!buf) {
         ui_error(op, "Unable to mmap file '%s' (%s).", strerror(errno));
         return NULL;
@@ -708,13 +725,13 @@ static BackupInfo *read_backup_log_file(Options *op)
 
     b = nvalloc(sizeof(BackupInfo));
 
-    b->version = get_next_line(buf, &c);
+    b->version = get_next_line(buf, &c, buf, length);
     if (!b->version || !c) goto parse_error;
     
     percent = (float) (c - buf) / (float) stat_buf.st_size;
     ui_status_update(op, percent, NULL);
 
-    b->description = get_next_line(c, &c);
+    b->description = get_next_line(c, &c, buf, length);
     if (!b->description || !c) goto parse_error;
 
     b->n = 0;
@@ -728,7 +745,7 @@ static BackupInfo *read_backup_log_file(Options *op)
         
         /* read and parse the next line */
 
-        line = get_next_line(c, &c);
+        line = get_next_line(c, &c, buf, length);
         if (!line) break;
 
         if (!parse_first_line(line, &num, &filename)) goto parse_error;
@@ -749,7 +766,8 @@ static BackupInfo *read_backup_log_file(Options *op)
         switch(e->num) {
 
         case INSTALLED_FILE:
-            if ((line = get_next_line(c, &c)) == NULL) goto parse_error;
+            line = get_next_line(c, &c, buf, length);
+            if (line == NULL) goto parse_error;
             line_num++;
 
             if (!parse_crc(line, &e->crc)) goto parse_error;
@@ -758,7 +776,8 @@ static BackupInfo *read_backup_log_file(Options *op)
             break;
 
         case INSTALLED_SYMLINK:
-            if ((line = get_next_line(c, &c)) == NULL) goto parse_error;
+            line = get_next_line(c, &c, buf, length);
+            if (line == NULL) goto parse_error;
             line_num++;
             
             e->target = line;
@@ -766,12 +785,14 @@ static BackupInfo *read_backup_log_file(Options *op)
             break;
             
         case BACKED_UP_SYMLINK:
-            if ((line = get_next_line(c, &c)) == NULL) goto parse_error;
+            line = get_next_line(c, &c, buf, length);
+            if (line == NULL) goto parse_error;
             line_num++;
             
             e->target = line;
 
-            if ((line = get_next_line(c, &c)) == NULL) goto parse_error;
+            line = get_next_line(c, &c, buf, length);
+            if (line == NULL) goto parse_error;
             line_num++;
 
             if (!parse_mode_uid_gid(line, &e->mode, &e->uid, &e->gid))
@@ -783,7 +804,8 @@ static BackupInfo *read_backup_log_file(Options *op)
         default:
             if (num < BACKED_UP_FILE_NUM) goto parse_error;
             
-            if ((line = get_next_line(c, &c)) == NULL) goto parse_error;
+            line = get_next_line(c, &c, buf, length);
+            if (line == NULL) goto parse_error;
             line_num++;
 
             if (!parse_crc_mode_uid_gid(line, &e->crc, &e->mode,
@@ -975,34 +997,51 @@ static int check_backup_log_entries(Options *op, BackupInfo *b)
  * XXX we should probably check the file permissions of BACKUP_LOG.
  */
 
-char *get_installed_driver_version_and_descr(Options *op, int *major,
-                                             int *minor, int *patch)
+int get_installed_driver_version_and_descr(Options *op,
+                                           char **pVersion, char **pDescr)
 {
     struct stat stat_buf;
     char *c, *version = NULL, *descr = NULL, *buf = NULL;
-    int fd = -1;
+    char *version_line = NULL;
+    int length, fd = -1;
+    int ret = FALSE;
 
     if ((fd = open(BACKUP_LOG, O_RDONLY)) == -1) goto done;
     
     if (fstat(fd, &stat_buf) == -1) goto done;
     
     /* map the file */
+
+    length = stat_buf.st_size;
     
-    buf = mmap(0, stat_buf.st_size, PROT_READ, MAP_FILE | MAP_SHARED, fd, 0);
+    buf = mmap(0, length, PROT_READ, MAP_FILE | MAP_SHARED, fd, 0);
     if (!buf) goto done;
     
-    version = get_next_line(buf, &c);
+    version_line = get_next_line(buf, &c, buf, length);
+    if (!version_line) goto done;
+
+    version = extract_version_string(version_line);
+
     if (!version) goto done;
-
-    if (!nvid_version(version, major, minor, patch)) goto done;
     
-    descr = get_next_line(c, NULL);
+    descr = get_next_line(c, NULL, buf, length);
 
+    if (!descr) goto done;
+
+    *pVersion = strdup(version);
+    *pDescr = strdup(descr);
+    
+    ret = TRUE;
+    
  done:
-    if (version) free(version);
+    nvfree(version_line);
+    nvfree(version);
+    nvfree(descr);
+
     if (buf) munmap(buf, stat_buf.st_size);
     if (fd != -1) close(fd);
-    return descr;
+    
+    return ret;
 
 } /* get_installed_driver_version_and_descr() */
 
@@ -1024,30 +1063,30 @@ char *get_installed_driver_version_and_descr(Options *op, int *major,
 
 int check_for_existing_driver(Options *op, Package *p)
 {
-    int major, minor, patch;
-    char *descr;
+    char *descr = NULL;
+    char *version = NULL;
+    int ret = FALSE;
+    int localRet;
 
     if (!check_for_existing_rpms(op)) return FALSE;
 
-    descr = get_installed_driver_version_and_descr(op, &major, &minor, &patch);
+    localRet = get_installed_driver_version_and_descr(op, &version, &descr);
 
     if (op->kernel_module_only) {
-        if (!descr) {
+        if (!localRet) {
             ui_error(op, "No NVIDIA driver is currently installed; the "
                      "'--kernel-module-only' option can only be used "
                      "to install the NVIDIA kernel module on top of an "
                      "existing driver installation.");
             return FALSE;
         } else {
-            if ((p->major != major) ||
-                (p->minor != minor) ||
-                (p->patch != patch)) {
+            if (strcmp(p->version, version) != 0) {
                 ui_error(op, "The '--kernel-module-only' option can only be "
                          "used to install a kernel module on top of an "
                          "existing driver installation of the same driver "
                          "version.  The existing driver installation is "
-                         "%d.%d-%d, but the kernel module is %d.%d-%d\n",
-                         major, minor, patch, p->major, p->minor, p->patch);
+                         "%s, but the kernel module is %s.\n",
+                         version, p->version);
                 return FALSE;
             } else {
                 return TRUE;
@@ -1055,7 +1094,12 @@ int check_for_existing_driver(Options *op, Package *p)
         }
     }
     
-    if (!descr) return TRUE;
+    /* no existing driver -- it is fine to continue with installation */
+
+    if (!localRet) {
+        ret = TRUE;
+        goto done;
+    }
 
     /*
      * XXX we could do a comparison, here, to check that the Package
@@ -1066,19 +1110,25 @@ int check_for_existing_driver(Options *op, Package *p)
      */
     
     if (!ui_yes_no(op, TRUE, "There appears to already be a driver installed "
-                   "on your system (version: %d.%d-%d).  As part of "
-                   "installing this driver (version: %d.%d-%d), the existing "
+                   "on your system (version: %s).  As part of "
+                   "installing this driver (version: %s), the existing "
                    "driver will be uninstalled.  Are you sure you want to "
                    "continue? ('no' will abort installation)",
-                   major, minor, patch, p->major, p->minor, p->patch)) {
+                   version, p->version)) {
         
         free(descr);
         ui_log(op, "Installation aborted.");
         return FALSE;
     }
     
-    free(descr);
-    return TRUE;
+    ret = TRUE;
+
+ done:
+
+    nvfree(descr);
+    nvfree(version);
+    
+    return ret;
 
 } /* check_for_existing_driver() */
 
@@ -1094,11 +1144,12 @@ int check_for_existing_driver(Options *op, Package *p)
 
 int uninstall_existing_driver(Options *op, const int interactive)
 {
-    int major, minor, patch, ret;
-    char *descr;
+    int ret;
+    char *descr = NULL;
+    char *version = NULL;
     
-    descr = get_installed_driver_version_and_descr(op, &major, &minor, &patch);
-    if (!descr) {
+    ret = get_installed_driver_version_and_descr(op, &version, &descr);
+    if (!ret) {
         if (interactive) {
             ui_message(op, "There is no NVIDIA driver currently installed.");
         }
@@ -1109,17 +1160,18 @@ int uninstall_existing_driver(Options *op, const int interactive)
 
     if (ret) {
         if (interactive) {
-            ui_message(op, "Uninstallation of existing driver: %s (%d.%d-%d) "
-                       "is complete.", descr, major, minor, patch);
+            ui_message(op, "Uninstallation of existing driver: %s (%s) "
+                       "is complete.", descr, version);
         } else {
-            ui_log(op, "Uninstallation of existing driver: %s (%d.%d-%d) "
-                   "is complete.", descr, major, minor, patch);
+            ui_log(op, "Uninstallation of existing driver: %s (%s) "
+                   "is complete.", descr, version);
         }
     } else {
         ui_error(op, "Uninstallation failed.");
     }
     
-    free(descr);
+    nvfree(descr);
+    nvfree(version);
 
     return TRUE;
 
@@ -1134,19 +1186,21 @@ int uninstall_existing_driver(Options *op, const int interactive)
 
 int report_driver_information(Options *op)
 {
-    int major, minor, patch;
-    char *descr;
-
-    descr = get_installed_driver_version_and_descr(op, &major, &minor, &patch);
-    if (!descr) {
+    char *descr, *version;
+    int ret;
+    
+    ret = get_installed_driver_version_and_descr(op, &version, &descr);
+    if (!ret) {
         ui_message(op, "There is no NVIDIA driver currently installed.");
         return FALSE;
     }
 
     ui_message(op, "The currently installed driver is: '%s' "
-               "(version: %d.%d-%d).", descr, major, minor, patch);
+               "(version: %s).", descr, version);
     
-    free(descr);
+    nvfree(descr);
+    nvfree(version);
+
     return TRUE;
     
 } /* report_driver_information() */
@@ -1321,3 +1375,61 @@ static int sanity_check_backup_log_entries(Options *op, BackupInfo *b)
     return ret;
     
 } /* sanity_check_backup_log_entries */
+
+
+
+/*
+ * create_backwards_compatible_version_string() - given the version
+ * string 'str', generate a version string to write to the backup log
+ * file that can be read by older nvidia-installers that assumed the
+ * X.Y-ZZZZ version format.
+ *
+ * Fortunately, old nvidia-installers' version parsing didn't care if
+ * there were extra digits beyond the four Z's (e.g., it could be
+ * X.Y-ZZZZZ, or X.Y-ZZZZZZZZ); they would just look for the first 4
+ * Z's if they needed to parse the version.
+ *
+ * So, the strategy is to take the new version string, remove any
+ * periods, and print that out as the old style version, followed by
+ * the real version string in parenthesis; e.g.,
+ *
+ *  "1.0-105917 (105.9.17)"
+ *
+ * In this way, an old nvidia-installer will atleast be able to parse
+ * the string, even though it may not understand it, but a new
+ * nvidia-installer can be smart and pull out the new version string.
+ */
+
+static char *create_backwards_compatible_version_string(const char *str)
+{
+    char *version, *scratch, *s, *t;
+    int len;
+
+    /*
+     * create a scratch string by duping the passed in str, but
+     * collapse the scratch string by only retaining the digits; e.g.,
+     * "105.9.17" --> "105917"
+     */
+    
+    scratch = nvstrdup(str);
+
+    for (s = t = scratch; *t; t++) {
+        if (isdigit(*t)) {
+            *s++ = *t;
+        }
+    }
+    
+    *s = '\0';
+
+    /* allocate a new string to contain the result */
+
+    len = strlen(str);
+    version = nvalloc((len * 2) + 16);
+    
+    sprintf(version, "1.0-%s (%s)", scratch, str);
+    
+    nvfree(scratch);
+
+    return version;
+    
+} /* create_backwards_compatible_version_string() */
