@@ -582,8 +582,9 @@ static ConflictingFileInfo __opengl_libs[] = {
     { "libGL.",         6,  /* strlen("libGL.") */         NULL },
     { "libnvidia-tls.", 14, /* strlen("libnvidia-tls.") */ NULL },
     { "libGLwrapper.",  13, /* strlen("libGLwrapper.") */  NULL },
+    { "libnvidia-cfg.", 14, /* strlen("libnvidia-cfg.") */ NULL },
     { "libcuda.",       8,  /* strlen("libcuda.") */       NULL },
-    { NULL, 0 }
+    { NULL,             0,                                 NULL }
 };
 
 /*
