@@ -539,6 +539,13 @@ int set_destinations(Options *op, Package *p)
             path = p->entries[i].path;
             break;
 
+        case FILE_TYPE_UTILITY_LIB:
+        case FILE_TYPE_UTILITY_LIB_SYMLINK:
+            prefix = op->utility_prefix;
+            dir = op->utility_libdir;
+            path = "";
+            break;
+
         case FILE_TYPE_LIBGL_LA:
             if (p->entries[i].flags & FILE_CLASS_COMPAT32) {
                 prefix = op->compat32_prefix;
@@ -1857,7 +1864,7 @@ int set_security_context(Options *op, const char *filename)
 
 /*
  * get_default_prefixes_and_paths() - assign the default prefixes and
- * paths depending on the architecture, distribution and the Xorg
+ * paths depending on the architecture, distribution and the X.Org
  * version installed on the system.
  */
 
