@@ -78,12 +78,13 @@ int install_from_cwd(Options *op)
     Package *p;
     CommandList *c;
     const char *msg;
+    int ret;
     int ran_pre_install_hook = FALSE;
 
     static const char edit_your_xf86config[] =
         "Please update your XF86Config or xorg.conf file as "
         "appropriate; see the file /usr/share/doc/"
-        "NVIDIA_GLX-1.0/README for details.";
+        "NVIDIA_GLX-1.0/README.txt for details.";
 
     static const char suse_edit_your_xf86config[] =
         "On SuSE Linux/United Linux please use SaX2 now to enable "
@@ -98,6 +99,13 @@ int install_from_cwd(Options *op)
     
     ui_set_title(op, "%s (%s)", p->description, p->version);
     
+    /* 
+     * warn the user if "legacy" GPUs are installed in this system
+     * and if no supported GPU is found, at all.
+     */
+
+    check_for_nvidia_graphics_devices(op, p);
+
     /* check that we are not running any X server */
 
     if (!check_for_running_x(op)) goto failed;
@@ -232,10 +240,6 @@ int install_from_cwd(Options *op)
 
     run_distro_hook(op, "post-install");
 
-    /* 
-     * XXX IMPLEMENT ME: generate an XF86Config file?
-     */
-  
     /*
      * check that everything is installed properly (post-install
      * sanity check)
@@ -248,19 +252,42 @@ int install_from_cwd(Options *op)
     
     /* done */
 
-    if ((op->distro == SUSE) || (op->distro == UNITED_LINUX)) {
-        msg = suse_edit_your_xf86config;
-    } else {
-        msg = edit_your_xf86config;
-    }
-    
-    if (op->kernel_module_only) {
+    if (op->kernel_module_only || op->no_nvidia_xconfig_question) {
+
         ui_message(op, "Installation of the kernel module for the %s "
                    "(version %s) is now complete.",
                    p->description, p->version);
     } else {
-        ui_message(op, "Installation of the %s (version: %s) is now "
-                   "complete.  %s", p->description, p->version, msg);
+        
+        /* ask the user if they would like to run nvidia-xconfig */
+        
+        ret = ui_yes_no(op, op->run_nvidia_xconfig,
+                        "Would you like to run the nvidia-xconfig utility "
+                        "to automatically update your X configuration file "
+                        "so that the NVIDIA X driver will be used when you "
+                        "restart X?  Any pre-existing X configuration "
+                        "file will be backed up.");
+        
+        if (ret) {
+            ret = run_nvidia_xconfig(op);
+        }
+        
+        if (ret) {
+            ui_message(op, "Your X configuration file has been successfully "
+                       "updated.  Installation of the %s (version: %s) is now "
+                       "complete.", p->description, p->version);
+        } else {
+            
+            if ((op->distro == SUSE) || (op->distro == UNITED_LINUX)) {
+                msg = suse_edit_your_xf86config;
+            } else {
+                msg = edit_your_xf86config;
+            }
+            
+            ui_message(op, "Installation of the %s (version: %s) is now "
+                       "complete.  %s", p->description,
+                       p->version, msg);
+        }
     }
     
     return TRUE;
@@ -623,6 +650,8 @@ static Package *parse_manifest (Options *op)
                 p->entries[n].flags |= FILE_TYPE_XLIB_SHARED_LIB;
             else if (strcmp(flag, "TLS_LIB") == 0)
                 p->entries[n].flags |= FILE_TYPE_TLS_LIB;
+            else if (strcmp(flag, "UTILITY_LIB") == 0)
+                p->entries[n].flags |= FILE_TYPE_UTILITY_LIB;
             else if (strcmp(flag, "DOCUMENTATION") == 0)
                 p->entries[n].flags |= FILE_TYPE_DOCUMENTATION;
             else if (strcmp(flag, "MANPAGE") == 0)
@@ -633,6 +662,8 @@ static Package *parse_manifest (Options *op)
                 p->entries[n].flags |= FILE_TYPE_XLIB_SYMLINK;
             else if (strcmp(flag, "TLS_SYMLINK") == 0)
                 p->entries[n].flags |= FILE_TYPE_TLS_SYMLINK;
+            else if (strcmp(flag, "UTILITY_LIB_SYMLINK") == 0)
+                p->entries[n].flags |= FILE_TYPE_UTILITY_LIB_SYMLINK;
             else if (strcmp(flag, "INSTALLER_BINARY") == 0)
                 p->entries[n].flags |= FILE_TYPE_INSTALLER_BINARY;
             else if (strcmp(flag, "UTILITY_BINARY") == 0)
