@@ -34,6 +34,8 @@
 #include <dirent.h>
 #include <fcntl.h>
 #include <sys/mman.h>
+#include <string.h>
+#include <limits.h>
 #include <fts.h>
 
 #include "nvidia-installer.h"
@@ -74,9 +76,9 @@ static const char install_your_kernel_source[] =
 "Please make sure you have installed the kernel source files for "
 "your kernel and that they are properly configured; on Red Hat "
 "Linux systems, for example, be sure you have the 'kernel-source' "
-"RPM installed.  If you know the correct kernel source files are "
-"installed, you may specify the kernel source path with the "
-"'--kernel-source-path' command line option.";
+"or 'kernel-devel' RPM installed.  If you know the correct kernel "
+"source files are installed, you may specify the kernel source "
+"path with the '--kernel-source-path' command line option.";
 
  
 
@@ -710,6 +712,16 @@ int test_kernel_module(Options *op, Package *p)
         nvfree(cmd);
     }
 
+    /*
+     * Likewise, we need to preload the i2c-core.ko kernel module to
+     * satisfy another dependency not resolved by `insmod`.
+     */
+    if (strncmp(get_kernel_name(op), "2.4", 3) != 0) {
+        cmd = nvstrcat(op->utils[MODPROBE], " -q i2c-core", NULL);
+        run_command(op, cmd, NULL, FALSE, 0, TRUE);
+        nvfree(cmd);
+    }
+
     cmd = nvstrcat(op->utils[INSMOD], " ",
                    p->kernel_module_build_directory, "/",
                    p->kernel_module_filename, NULL);
@@ -1491,7 +1503,6 @@ int check_cc_version(Options *op, Package *p)
 } /* check_cc_version() */
 
 
-
 /*
  * fbdev_check() - run the rivafb_sanity_check and the nvidiafb_sanity_check
  * conftests; if either test fails, print the error message from the test
