@@ -40,6 +40,11 @@
 #include <sys/mman.h>
 #include <dirent.h>
 #include <libgen.h>
+#include <pci/pci.h>
+
+#ifndef PCI_CLASS_DISPLAY_3D
+#define PCI_CLASS_DISPLAY_3D 0x302
+#endif
 
 #include "nvidia-installer.h"
 #include "user-interface.h"
@@ -47,10 +52,10 @@
 #include "files.h"
 #include "misc.h"
 #include "crc.h"
+#include "nvLegacy.h"
 
 static int check_symlink(Options*, const char*, const char*, const char*);
 static int check_file(Options*, const char*, const mode_t, const uint32);
-static char *find_system_util(const char *util);
 
 
 /*
@@ -970,9 +975,9 @@ int check_development_tools(Options *op, Package *p)
  * utility is returned.  On failure NULL is returned.
  */
 
-static char *find_system_util(const char *util)
+char *find_system_util(const char *util)
 {
-    char *buf, *path, *file, *x, *y;
+    char *buf, *path, *file, *x, *y, c;
     
     /* build the search path */
     
@@ -985,17 +990,19 @@ static char *find_system_util(const char *util)
 
     /* search the PATH for the utility */
 
-    for (x = y = path; *x; x++) {
-        if (*x == ':') {
+    for (x = y = path; ; x++) {
+        if (*x == ':' || *x == '\0') {
+            c = *x;
             *x = '\0';
             file = nvstrcat(y, "/", util, NULL);
-            *x = ':';
+            *x = c;
             if ((access(file, F_OK | X_OK)) == 0) {
                 nvfree(path);
                 return file;
             }
             nvfree(file);
             y = x + 1;
+            if (*x == '\0') break;
         }
     }
 
@@ -1025,6 +1032,8 @@ int continue_after_error(Options *op, const char *fmt, ...)
                     "error during installation: '%s'.  Continue anyway? "
                     "(\"no\" will abort)?", msg);
 
+    nvfree(msg);
+
     return ret;
 
 } /* continue_after_error() */
@@ -2253,6 +2262,92 @@ int check_for_running_x(Options *op)
 
 } /* check_for_running_x() */
 
+
+/*
+ * check_for_nvidia_graphics_devices() - check if there are supported
+ * NVIDIA graphics devices installed in this system. If one or more
+ * supported devices are found, the function returns TRUE, else it prints
+ * a warning message and returns FALSE. If legacy devices are detected
+ * in the system, a warning message is printed for each one.
+ */
+
+static void ignore_libpci_output(char *fmt, ...)
+{
+}
+
+int check_for_nvidia_graphics_devices(Options *op, Package *p)
+{
+    struct pci_access *pacc;
+    struct pci_dev *dev;
+    int i, found_supported_device = FALSE;
+    int found_vga_device = FALSE;
+    uint16 class;
+
+    pacc = pci_alloc();
+    if (!pacc) return TRUE;
+
+    pacc->error = ignore_libpci_output;
+    pacc->warning = ignore_libpci_output;
+    pci_init(pacc);
+    if (!pacc->methods) return TRUE;
+
+    pci_scan_bus(pacc);
+
+    for (dev = pacc->devices; dev != NULL; dev = dev->next) {
+        if ((pci_fill_info(dev, PCI_FILL_IDENT) & PCI_FILL_IDENT) == 0)
+            continue;
+
+        class = pci_read_word(dev, PCI_CLASS_DEVICE);
+
+        if ((class == PCI_CLASS_DISPLAY_VGA || class == PCI_CLASS_DISPLAY_3D) &&
+              (dev->vendor_id == 0x10de) /* NVIDIA */ &&
+              (dev->device_id >= 0x0020) /* TNT or later */) {
+            /*
+             * First check if this GPU is a "legacy" GPU; if it is, print a
+             * warning message and point the user to the NVIDIA Linux
+             * driver download page for.
+             */
+            int found_legacy_device = FALSE;
+            for (i = 0; i < sizeof(LegacyList) / sizeof(LEGACY_INFO); i++) {
+                if (dev->device_id == LegacyList[i].uiDevId) {
+                    ui_warn(op, "The NVIDIA %s GPU installed in this system is supported "
+                            "through the NVIDIA legacy Linux graphics drivers.  Please "
+                            "visit http://www.nvidia.com/object/unix.html for more "
+                            "information.  The %s NVIDIA Linux graphics driver will "
+                            "ignore this GPU.", LegacyList[i].AdapterString, p->version);
+                    found_legacy_device = TRUE;
+                }
+            }
+
+            if (!found_legacy_device) {
+                found_supported_device = TRUE;
+
+                if (class == PCI_CLASS_DISPLAY_VGA)
+                    found_vga_device = TRUE;
+            }
+        }
+    }
+
+    pci_cleanup(pacc);
+    if (!pacc->devices) return TRUE;
+
+    if (!found_supported_device) {
+        ui_warn(op, "You do not appear to have an NVIDIA GPU supported by the "
+                 "%s NVIDIA Linux graphics driver installed in this system.  "
+                 "For further details, please see the appendix SUPPORTED "
+                 "NVIDIA GRAPHICS CHIPS in the README available on the Linux "
+                 "driver download page at www.nvidia.com.", p->version);
+        return FALSE;
+    }
+
+    if (!found_vga_device)
+        op->no_nvidia_xconfig_question = TRUE;
+
+    return TRUE;
+
+} /* check_for_nvidia_graphics_devices() */
+
+
 /*
  * check_selinux() - check if selinux is available.
  * Sets the variable op->selinux_enabled.
@@ -2365,6 +2460,34 @@ int check_selinux(Options *op)
     return TRUE;
 } /* check_selinux */
 
+/*
+ * run_nvidia_xconfig() - run the `nvidia-xconfig` utility.  Without
+ * any options, this will just make sure the X config file uses the
+ * NVIDIA driver by default.
+ */
+
+int run_nvidia_xconfig(Options *op)
+{
+    int ret, bRet = TRUE;
+    char *data = NULL, *cmd;
+    
+    cmd = find_system_util("nvidia-xconfig");
+    
+    ret = run_command(op, cmd, &data, FALSE, FALSE, TRUE);
+    nvfree(cmd);
+    
+    if (ret != 0) {
+        ui_error(op, "Failed to run nvidia-xconfig:\n%s", data);
+        bRet = FALSE;
+    }
+    
+    nvfree(data);
+
+    return bRet;
+    
+} /* run_nvidia_xconfig() */
+
+
 /*
  * run_distro_hook() - run a distribution-provided hook script
  */
@@ -2416,6 +2539,7 @@ done:
     return ret;
 }
 
+
 /*
  * nv_format_text_rows() - this function breaks the given string str
  * into some number of rows, where each row is not longer than the
