@@ -88,7 +88,6 @@ void nvfree(char *s);
 char *nvstrtolower(char *s);
 char *nvstrcat(const char *str, ...);
 char *read_next_word (char *buf, char **e);
-char *assemble_string(const char *fmt, va_list ap);
 
 int check_euid(Options *op);
 int check_runlevel(Options *op);
@@ -98,6 +97,7 @@ char *get_next_line(char *buf, char **e, char *start, int length);
 int run_command(Options *op, const char *cmd, char **data,
                 int output, int status, int redirect);
 int read_text_file(const char *filename, char **buf);
+char *find_system_util(const char *util);
 int find_system_utils(Options *op);
 int find_module_utils(Options *op);
 int check_selinux(Options *op);
@@ -117,6 +117,8 @@ int is_symbolic_link_to(const char *path, const char *dest);
 Distribution get_distribution(Options *op);
 int check_for_running_x(Options *op);
 int check_for_modular_xorg(Options *op);
+int check_for_nvidia_graphics_devices(Options *op, Package *p);
+int run_nvidia_xconfig(Options *op);
 int run_distro_hook(Options *op, const char *hook);
 
 TextRows *nv_format_text_rows(const char *prefix, const char *buf,
