@@ -209,6 +209,7 @@ static Options *load_default_options(void)
 
     op->logging = TRUE; /* log by default */
     op->opengl_headers = TRUE; /* We now install our GL headers by default */
+    op->run_nvidia_xconfig = FALSE;
     op->selinux_option = SELINUX_DEFAULT;
 
     op->sigwinch_workaround = TRUE;
@@ -266,6 +267,7 @@ static void parse_commandline(int argc, char *argv[], Options *op)
             op->kernel_module_only = TRUE;
             op->no_kernel_module = FALSE; /* conflicts  */
             break;
+        case 'X': op->run_nvidia_xconfig = TRUE; break;
         case 's':
             op->silent = op->no_questions = op->accept_license = TRUE;
             op->ui_str = "none";
