@@ -141,7 +141,7 @@ static const NVGetoptOption __options[] = {
     { "x-prefix", X_PREFIX_OPTION, NVGETOPT_STRING_ARGUMENT,
       "The prefix under which the X components of the "
       "NVIDIA driver will be installed; the default is '" DEFAULT_X_PREFIX
-      "' unless nvidia-installer detects that Xorg >= 7.0 is installed, "
+      "' unless nvidia-installer detects that X.Org >= 7.0 is installed, "
       "in which case the default is '" XORG7_DEFAULT_X_PREFIX "'.  Only "
       "under rare circumstances should this option be used." },
 
@@ -412,6 +412,16 @@ static const NVGetoptOption __options[] = {
       "for distributing precompiled kernel interfaces in a local area "
       "network." },
 
+    { "run-nvidia-xconfig", 'X', 0,
+      "nvidia-installer can optionally invoke the nvidia-xconfig utility.  "
+      "This will update the system X configuration file so that the NVIDIA X "
+      "driver is used.  The pre-existing X configuration file will be backed "
+      "up.  At the end of installation, nvidia-installer will "
+      "ask the user if they wish to run nvidia-xconfig; the default "
+      "response is 'no'.  Use this option to make the default response "
+      "'yes'.  This is useful with the '--no-questions' or '--silent' "
+      "options, which assume the default values for all questions." },
+    
     { "force-selinux", FORCE_SELINUX_OPTION, NVGETOPT_STRING_ARGUMENT,
       "Linux installations using SELinux (Security-Enhanced Linux) "
       "require that the security type of all shared libraries be set "
